@@ -74,7 +74,7 @@ struct _WnckScreenPrivate
   int number;
   Window xroot;
   Screen *xscreen;
-  
+
   /* in map order */
   GList *mapped_windows;
   /* in stacking order */
@@ -100,19 +100,19 @@ struct _WnckScreenPrivate
   Pixmap bg_pixmap;
 
   char *wm_name;
-  
-  guint update_handler;  
+
+  guint update_handler;
 
 #ifdef HAVE_STARTUP_NOTIFICATION
   SnDisplay *sn_display;
 #endif
-  
+
   guint showing_desktop : 1;
-  
+
   guint vertical_workspaces : 1;
   _WnckLayoutCorner starting_corner;
   gint rows_of_workspaces;
-  gint columns_of_workspaces;  
+  gint columns_of_workspaces;
 
   /* if you add flags, be sure to set them
    * when we create the screen so we get an initial update
@@ -165,7 +165,7 @@ static void update_workspace_names    (WnckScreen      *screen);
 static void update_showing_desktop    (WnckScreen      *screen);
 
 static void queue_update            (WnckScreen      *screen);
-static void unqueue_update          (WnckScreen      *screen);              
+static void unqueue_update          (WnckScreen      *screen);
 static void do_update_now           (WnckScreen      *screen);
 
 static void emit_active_window_changed    (WnckScreen      *screen);
@@ -197,7 +197,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static void
 wnck_screen_init (WnckScreen *screen)
-{  
+{
   screen->priv = WNCK_SCREEN_GET_PRIVATE (screen);
 
   screen->priv->number = -1;
@@ -288,7 +288,7 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1, WNCK_TYPE_WORKSPACE);
-  
+
   /**
    * WnckScreen::window-stacking-changed:
    * @screen: the #WnckScreen which emitted the signal.
@@ -335,11 +335,11 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1, WNCK_TYPE_WINDOW);
-  
+
   /**
    * WnckScreen::workspace-created:
    * @screen: the #WnckScreen which emitted the signal.
-   * @space: the workspace that has been created. 
+   * @space: the workspace that has been created.
    *
    * Emitted when a #WnckWorkspace is created on @screen.
    */
@@ -351,11 +351,11 @@ wnck_screen_class_init (WnckScreenClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1, WNCK_TYPE_WORKSPACE);
-  
+
   /**
    * WnckScreen::workspace-destroyed:
    * @screen: the #WnckScreen which emitted the signal.
-   * @space: the workspace that has been destroyed. 
+   * @space: the workspace that has been destroyed.
    *
    * Emitted when a #WnckWorkspace is destroyed on @screen.
    */
@@ -512,7 +512,7 @@ wnck_screen_finalize (GObject *object)
   gpointer weak_pointer;
 
   screen = WNCK_SCREEN (object);
-  
+
   unqueue_update (screen);
 
   for (tmp = screen->priv->stacked_windows; tmp; tmp = tmp->next)
@@ -557,7 +557,7 @@ wnck_screen_finalize (GObject *object)
   sn_display_unref (screen->priv->sn_display);
   screen->priv->sn_display = NULL;
 #endif
-  
+
   G_OBJECT_CLASS (wnck_screen_parent_class)->finalize (object);
 }
 
@@ -566,35 +566,37 @@ static void
 sn_error_trap_push (SnDisplay *display,
                     Display   *xdisplay)
 {
-  gdk_error_trap_push ();
+  _wnck_error_trap_push ();
 }
 
 static void
 sn_error_trap_pop (SnDisplay *display,
                    Display   *xdisplay)
 {
-  gdk_error_trap_pop ();
+  _wnck_error_trap_pop ();
 }
 #endif /* HAVE_STARTUP_NOTIFICATION */
 
 static void
-wnck_screen_construct (WnckScreen *screen,
+wnck_screen_construct (Display    *display,
+                       WnckScreen *screen,
                        int         number)
 {
   /* Create the initial state of the screen. */
-  screen->priv->xroot = RootWindow (gdk_display, number);
-  screen->priv->xscreen = ScreenOfDisplay (gdk_display, number);
+  screen->priv->xroot = RootWindow (display, number);
+  screen->priv->xscreen = ScreenOfDisplay (display, number);
   screen->priv->number = number;
 
 #ifdef HAVE_STARTUP_NOTIFICATION
-  screen->priv->sn_display = sn_display_new (gdk_display,
+  screen->priv->sn_display = sn_display_new (display,
                                              sn_error_trap_push,
                                              sn_error_trap_pop);
 #endif
-  
+
   screen->priv->bg_pixmap = None;
-  
-  _wnck_select_input (screen->priv->xroot,
+
+  _wnck_select_input (screen->priv->xscreen,
+                      screen->priv->xroot,
                       PropertyChangeMask);
 
   screen->priv->need_update_workspace_list = TRUE;
@@ -607,16 +609,16 @@ wnck_screen_construct (WnckScreen *screen,
   screen->priv->need_update_bg_pixmap = TRUE;
   screen->priv->need_update_showing_desktop = TRUE;
   screen->priv->need_update_wm = TRUE;
-  
+
   queue_update (screen);
 }
 
 /**
  * wnck_screen_get:
  * @index: screen number, starting from 0.
- * 
+ *
  * Gets the #WnckScreen for a given screen on the default display.
- * 
+ *
  * Return value: (transfer none): the #WnckScreen for screen @index, or %NULL
  * if no such screen exists. The returned #WnckScreen is owned by libwnck and
  * must not be referenced or unreferenced.
@@ -624,22 +626,26 @@ wnck_screen_construct (WnckScreen *screen,
 WnckScreen*
 wnck_screen_get (int index)
 {
-  g_return_val_if_fail (gdk_display != NULL, NULL);
+  Display *display;
 
-  if (index >= ScreenCount (gdk_display))
+  display = _wnck_get_default_display ();
+
+  g_return_val_if_fail (display != NULL, NULL);
+
+  if (index >= ScreenCount (display))
     return NULL;
-  
+
   if (screens == NULL)
     {
-      screens = g_new0 (WnckScreen*, ScreenCount (gdk_display));
+      screens = g_new0 (WnckScreen*, ScreenCount (display));
       _wnck_event_filter_init ();
     }
-  
+
   if (screens[index] == NULL)
     {
       screens[index] = g_object_new (WNCK_TYPE_SCREEN, NULL);
-      
-      wnck_screen_construct (screens[index], index);
+
+      wnck_screen_construct (display, screens[index], index);
     }
 
   return screens[index];
@@ -648,8 +654,12 @@ wnck_screen_get (int index)
 WnckScreen*
 _wnck_screen_get_existing (int number)
 {
-  g_return_val_if_fail (gdk_display != NULL, NULL);
-  g_return_val_if_fail (number < ScreenCount (gdk_display), NULL);
+  Display *display;
+
+  display = _wnck_get_default_display ();
+
+  g_return_val_if_fail (display != NULL, NULL);
+  g_return_val_if_fail (number < ScreenCount (display), NULL);
 
   if (screens != NULL)
     return screens[number];
@@ -659,9 +669,9 @@ _wnck_screen_get_existing (int number)
 
 /**
  * wnck_screen_get_default:
- * 
+ *
  * Gets the default #WnckScreen on the default display.
- * 
+ *
  * Return value: (transfer none): the default #WnckScreen. The returned
  * #WnckScreen is owned by libwnck and must not be referenced or unreferenced.
  **/
@@ -670,7 +680,7 @@ wnck_screen_get_default (void)
 {
   int default_screen;
 
-  default_screen = DefaultScreen (gdk_display);
+  default_screen = DefaultScreen (_wnck_get_default_display ());
 
   return wnck_screen_get (default_screen);
 }
@@ -678,13 +688,13 @@ wnck_screen_get_default (void)
 /**
  * wnck_screen_get_for_root:
  * @root_window_id: an X window ID.
- * 
+ *
  * Gets the #WnckScreen for the root window at @root_window_id, or
  * %NULL if no #WnckScreen exists for this root window.
  *
  * This function does not work if wnck_screen_get() was not called for the
  * sought #WnckScreen before, and returns %NULL.
- * 
+ *
  * Return value: (transfer none): the #WnckScreen for the root window at
  * @root_window_id, or %NULL. The returned #WnckScreen is owned by libwnck and
  * must not be referenced or unreferenced.
@@ -693,16 +703,19 @@ WnckScreen*
 wnck_screen_get_for_root (gulong root_window_id)
 {
   int i;
-  
+  Display *display;
+
   if (screens == NULL)
     return NULL;
 
   i = 0;
-  while (i < ScreenCount (gdk_display))
+  display = _wnck_get_default_display ();
+
+  while (i < ScreenCount (display))
     {
       if (screens[i] != NULL && screens[i]->priv->xroot == root_window_id)
         return screens[i];
-      
+
       ++i;
     }
 
@@ -712,10 +725,10 @@ wnck_screen_get_for_root (gulong root_window_id)
 /**
  * wnck_screen_get_number:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the index of @screen on the display to which it belongs. The first
  * #WnckScreen has an index of 0.
- * 
+ *
  * Return value: the index of @space on @screen, or -1 on errors.
  *
  * Since: 2.20
@@ -731,10 +744,10 @@ wnck_screen_get_number (WnckScreen *screen)
 /**
  * wnck_screen_get_workspaces:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the list of #WnckWorkspace on @screen. The list is ordered: the
  * first element in the list is the first #WnckWorkspace, etc..
- * 
+ *
  * Return value: (element-type WnckWorkspace) (transfer none): the list of
  * #WnckWorkspace on @screen. The list should not be modified nor freed, as it
  * is owned by @screen.
@@ -745,7 +758,7 @@ GList*
 wnck_screen_get_workspaces (WnckScreen *screen)
 {
   g_return_val_if_fail (WNCK_IS_SCREEN (screen), NULL);
-  
+
   return screen->priv->workspaces;
 }
 
@@ -753,9 +766,9 @@ wnck_screen_get_workspaces (WnckScreen *screen)
  * wnck_screen_get_workspace:
  * @screen: a #WnckScreen.
  * @workspace: a workspace index, starting from 0.
- * 
+ *
  * Gets the #WnckWorkspace numbered @workspace on @screen.
- * 
+ *
  * Return value: (transfer none): the #WnckWorkspace numbered @workspace on
  * @screen, or %NULL if no such workspace exists. The returned #WnckWorkspace
  * is owned by libwnck and must not be referenced or unreferenced.
@@ -767,7 +780,7 @@ wnck_screen_get_workspace (WnckScreen *screen,
   GList *list;
 
   g_return_val_if_fail (WNCK_IS_SCREEN (screen), NULL);
-  
+
   /* We trust this function with property-provided numbers, it
    * must reliably return NULL on bad data
    */
@@ -775,54 +788,18 @@ wnck_screen_get_workspace (WnckScreen *screen,
 
   if (list == NULL)
     return NULL;
-  
+
   return WNCK_WORKSPACE (list->data);
 }
 
-/**
- * wnck_screen_get_workspace_index:
- * @screen: a #WnckScreen.
- * @space: a #WnckWorkspace.
- * 
- * Gets the index of @space on @screen. The first #WnckWorkspace has an
- * index of 0. See also wnck_workspace_get_number().
- * 
- * Return value: the index of @space on @screen, or -1 on errors.
- *
- * Since: 2.14
- * Deprecated:2.20: Use wnck_workspace_get_number() instead.
- **/
-int
-wnck_screen_get_workspace_index (WnckScreen    *screen,
-                                 WnckWorkspace *space)
-{
-  GList *tmp;
-  int i;
-
-  g_return_val_if_fail (WNCK_IS_SCREEN (screen), -1);
-
-  i = 0;
-  tmp = screen->priv->workspaces;
-  while (tmp != NULL)
-    {
-      if (tmp->data == space)
-        return i;
-
-      ++i;
-
-      tmp = tmp->next;
-    }
-  return -1; /* compiler warnings */
-}
-
 /**
  * wnck_screen_get_active_workspace:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the active #WnckWorkspace on @screen. May return %NULL sometimes,
  * if libwnck is in a weird state due to the asynchronous nature of the
  * interaction with the window manager.
- * 
+ *
  * Return value: (transfer none): the active #WnckWorkspace on @screen, or
  * %NULL. The returned #WnckWorkspace is owned by libwnck and must not be
  * referenced or unreferenced.
@@ -835,79 +812,13 @@ wnck_screen_get_active_workspace (WnckScreen *screen)
   return screen->priv->active_workspace;
 }
 
-/**
- * wnck_screen_get_workspace_neighbor:
- * @screen: a #WnckScreen.
- * @space: a #WnckWorkspace.
- * @direction: direction in which to search the neighbor.
- * 
- * Gets the neighbor #WnckWorkspace of @space in the @direction direction on
- * @screen.
- * 
- * Return value: (transfer none): the neighbor #WnckWorkspace of @space in the
- * @direction direction on @screen, or %NULL if no such neighbor #WnckWorkspace
- * exists. The returned #WnckWorkspace is owned by libwnck and must not be
- * referenced or unreferenced.
- *
- * Since: 2.14
- * Deprecated:2.20: Use wnck_workspace_get_neighbor() instead.
- **/
-WnckWorkspace*
-wnck_screen_get_workspace_neighbor (WnckScreen         *screen,
-                                    WnckWorkspace      *space,
-                                    WnckMotionDirection direction)
-{
-  WnckWorkspaceLayout layout;
-  int i, space_index;
-
-  g_return_val_if_fail (WNCK_IS_SCREEN (screen), NULL);
-
-  space_index = wnck_screen_get_workspace_index (screen, space);
-
-  wnck_screen_calc_workspace_layout (screen, -1,
-                                     space_index, &layout);
-
-  switch (direction)
-    {
-    case WNCK_MOTION_LEFT:
-      layout.current_col -= 1;
-      break;
-    case WNCK_MOTION_RIGHT:
-      layout.current_col += 1;
-      break;
-    case WNCK_MOTION_UP:
-      layout.current_row -= 1;
-      break;
-    case WNCK_MOTION_DOWN:
-      layout.current_row += 1;
-      break;
-    }
-
-  if (layout.current_col < 0)
-    layout.current_col = 0;
-  if (layout.current_col >= layout.cols)
-    layout.current_col = layout.cols - 1;
-  if (layout.current_row < 0)
-    layout.current_row = 0;
-  if (layout.current_row >= layout.rows)
-    layout.current_row = layout.rows - 1;
-
-  i = layout.grid[layout.current_row * layout.cols + layout.current_col];
-
-  if (i < 0)
-    i = space_index;
-
-  wnck_screen_free_workspace_layout (&layout);
-  return wnck_screen_get_workspace (screen, i);
-}
-
 /**
  * wnck_screen_get_active_window:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the active #WnckWindow on @screen. May return %NULL sometimes, since
  * not all window managers guarantee that a window is always active.
- * 
+ *
  * Return value: (transfer none): the active #WnckWindow on @screen, or %NULL.
  * The returned #WnckWindow is owned by libwnck and must not be referenced or
  * unreferenced.
@@ -923,11 +834,11 @@ wnck_screen_get_active_window (WnckScreen *screen)
 /**
  * wnck_screen_get_previously_active_window:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the previously active #WnckWindow on @screen. May return %NULL
  * sometimes, since not all window managers guarantee that a window is always
  * active.
- * 
+ *
  * Return value: (transfer none): the previously active #WnckWindow on @screen,
  * or %NULL. The returned #WnckWindow is owned by libwnck and must not be
  * referenced or unreferenced.
@@ -945,12 +856,12 @@ wnck_screen_get_previously_active_window (WnckScreen *screen)
 /**
  * wnck_screen_get_windows:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the list of #WnckWindow on @screen. The list is not in a defined
  * order, but should be "stable" (windows should not be reordered in it).
  * However, the stability of the list is established by the window manager, so
  * don't blame libwnck if it breaks down.
- * 
+ *
  * Return value: (element-type WnckWindow) (transfer none): the list of
  * #WnckWindow on @screen, or %NULL if there is no window on @screen. The list
  * should not be modified nor freed, as it is owned by @screen.
@@ -966,9 +877,9 @@ wnck_screen_get_windows (WnckScreen *screen)
 /**
  * wnck_screen_get_windows_stacked:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the list of #WnckWindow on @screen in bottom-to-top order.
- * 
+ *
  * Return value: (element-type WnckWindow) (transfer none): the list of
  * #WnckWindow in stacking order on @screen, or %NULL if there is no window on
  * @screen. The list should not be modified nor freed, as it is owned by
@@ -985,10 +896,10 @@ wnck_screen_get_windows_stacked (WnckScreen *screen)
 /**
  * _wnck_screen_get_gdk_screen:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the <classname>GdkScreen</classname referring to the same screen as
  * @screen.
- * 
+ *
  * Return value: the <classname>GdkScreen</classname referring to the same
  * screen as @screen.
  **/
@@ -1004,7 +915,7 @@ _wnck_screen_get_gdk_screen (WnckScreen *screen)
 /**
  * wnck_screen_force_update:
  * @screen: a #WnckScreen.
- * 
+ *
  * Synchronously and immediately updates the list of #WnckWindow on @screen.
  * This bypasses the standard update mechanism, where the list of #WnckWindow
  * is updated in the idle loop.
@@ -1025,9 +936,9 @@ wnck_screen_force_update (WnckScreen *screen)
 /**
  * wnck_screen_get_workspace_count:
  * @screen: a #WnckScreen.
- * 
+ *
  * Gets the number of #WnckWorkspace on @screen.
- * 
+ *
  * Return value: the number of #WnckWorkspace on @screen.
  **/
 int
@@ -1042,7 +953,7 @@ wnck_screen_get_workspace_count (WnckScreen *screen)
  * wnck_screen_change_workspace_count:
  * @screen: a #WnckScreen.
  * @count: the number of #WnckWorkspace to request.
- * 
+ *
  * Asks the window manager to change the number of #WnckWorkspace on @screen.
  *
  * Since: 2.2
@@ -1052,10 +963,10 @@ wnck_screen_change_workspace_count (WnckScreen *screen,
                                     int         count)
 {
   XEvent xev;
-  
+
   g_return_if_fail (WNCK_IS_SCREEN (screen));
   g_return_if_fail (count >= 1);
-  
+
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
   xev.xclient.window = screen->priv->xroot;
@@ -1064,7 +975,7 @@ wnck_screen_change_workspace_count (WnckScreen *screen,
   xev.xclient.message_type = _wnck_atom_get ("_NET_NUMBER_OF_DESKTOPS");
   xev.xclient.format = 32;
   xev.xclient.data.l[0] = count;
-  
+
   _wnck_error_trap_push ();
   XSendEvent (DisplayOfScreen (screen->priv->xscreen),
               screen->priv->xroot,
@@ -1154,7 +1065,7 @@ _wnck_screen_process_property_notify (WnckScreen *screen,
  * @screen: a #WnckScreen.
  * @num_workspaces: the number of #WnckWorkspace on @screen, or -1 to let
  * wnck_screen_calc_workspace_layout() find this number.
- * @space_index: the index of a #Workspace.
+ * @space_index: the index of a #WnckWorkspace.
  * @layout: return location for the layout of #WnckWorkspace with additional
  * information.
  *
@@ -1341,7 +1252,7 @@ wnck_screen_calc_workspace_layout (WnckScreen          *screen,
             }
         }
       break;
-    } 
+    }
 
   current_row = 0;
   current_col = 0;
@@ -1447,7 +1358,7 @@ lists_equal (GList *a,
     {
       if (a_iter->data != b_iter->data)
         return FALSE;
-      
+
       a_iter = a_iter->next;
       b_iter = b_iter->next;
     }
@@ -1488,7 +1399,7 @@ arrays_contain_same_windows (Window *a,
 
   if (a_len == 0)
     return TRUE; /* both are empty */
-  
+
   a_tmp = g_new (Window, a_len);
   b_tmp = g_new (Window, b_len);
 
@@ -1502,7 +1413,7 @@ arrays_contain_same_windows (Window *a,
 
   g_free (a_tmp);
   g_free (b_tmp);
-  
+
   return result;
 }
 
@@ -1528,26 +1439,28 @@ update_client_list (WnckScreen *screen)
   gboolean active_changed;
   gboolean stack_changed;
   gboolean list_changed;
-  
+
   g_return_if_fail (reentrancy_guard == 0);
-  
+
   if (!screen->priv->need_update_stack_list)
     return;
 
   ++reentrancy_guard;
-  
+
   screen->priv->need_update_stack_list = FALSE;
-  
+
   stack = NULL;
   stack_length = 0;
-  _wnck_get_window_list (screen->priv->xroot,
+  _wnck_get_window_list (screen->priv->xscreen,
+                         screen->priv->xroot,
                          _wnck_atom_get ("_NET_CLIENT_LIST_STACKING"),
                          &stack,
                          &stack_length);
 
   mapping = NULL;
   mapping_length = 0;
-  _wnck_get_window_list (screen->priv->xroot,
+  _wnck_get_window_list (screen->priv->xscreen,
+                         screen->priv->xroot,
                          _wnck_atom_get ("_NET_CLIENT_LIST"),
                          &mapping,
                          &mapping_length);
@@ -1561,7 +1474,7 @@ update_client_list (WnckScreen *screen)
       --reentrancy_guard;
       return;
     }
-  
+
   created = NULL;
   closed = NULL;
   created_apps = NULL;
@@ -1570,7 +1483,7 @@ update_client_list (WnckScreen *screen)
   closed_class_groups = NULL;
 
   new_hash = g_hash_table_new (NULL, NULL);
-  
+
   new_list = NULL;
   i = 0;
   while (i < mapping_length)
@@ -1585,8 +1498,8 @@ update_client_list (WnckScreen *screen)
           WnckApplication *app;
 	  const char *res_class;
 	  WnckClassGroup *class_group;
-          
-          window = _wnck_window_create (mapping[i], 
+
+          window = _wnck_window_create (mapping[i],
                                         screen,
                                         screen->priv->window_order++);
 
@@ -1595,14 +1508,14 @@ update_client_list (WnckScreen *screen)
 	  /* Application */
 
           leader = wnck_window_get_group_leader (window);
-          
+
           app = wnck_application_get (leader);
           if (app == NULL)
             {
               app = _wnck_application_create (leader, screen);
               created_apps = g_list_prepend (created_apps, app);
             }
-          
+
           _wnck_application_add_window (app, window);
 
 	  /* Class group */
@@ -1622,10 +1535,10 @@ update_client_list (WnckScreen *screen)
       new_list = g_list_prepend (new_list, window);
 
       g_hash_table_insert (new_hash, window, window);
-      
+
       ++i;
     }
-      
+
   /* put list back in order */
   new_list = g_list_reverse (new_list);
 
@@ -1641,7 +1554,7 @@ update_client_list (WnckScreen *screen)
         {
           WnckApplication *app;
 	  WnckClassGroup *class_group;
-          
+
           closed = g_list_prepend (closed, window);
 
 	  /* Remove from the app */
@@ -1660,7 +1573,7 @@ update_client_list (WnckScreen *screen)
           if (wnck_class_group_get_windows (class_group) == NULL)
             closed_class_groups = g_list_prepend (closed_class_groups, class_group);
         }
-      
+
       tmp = tmp->next;
     }
 
@@ -1678,16 +1591,16 @@ update_client_list (WnckScreen *screen)
       g_assert (window != NULL);
 
       new_stack_list = g_list_prepend (new_stack_list, window);
-      
+
       ++i;
     }
-  
+
   g_free (stack);
   g_free (mapping);
-      
+
   /* put list back in order */
   new_stack_list = g_list_reverse (new_stack_list);
-  
+
   /* Now new_stack_list becomes screen->priv->stack_windows, new_list
    * becomes screen->priv->mapped_windows, and we emit the opened/closed
    * signals as appropriate
@@ -1705,7 +1618,7 @@ update_client_list (WnckScreen *screen)
       g_assert (created_class_groups == NULL);
       g_assert (closed_class_groups == NULL);
       g_list_free (new_stack_list);
-      g_list_free (new_list);      
+      g_list_free (new_list);
       --reentrancy_guard;
       return;
     }
@@ -1745,7 +1658,7 @@ update_client_list (WnckScreen *screen)
         {
           set_previously_active_window (screen, NULL);
         }
-      
+
       if (window == screen->priv->active_window)
         {
           set_previously_active_window (screen, screen->priv->active_window);
@@ -1767,7 +1680,7 @@ update_client_list (WnckScreen *screen)
 
   if (active_changed)
     emit_active_window_changed (screen);
-  
+
   /* Now free the closed windows */
   for (tmp = closed; tmp; tmp = tmp->next)
     _wnck_window_destroy (WNCK_WINDOW (tmp->data));
@@ -1808,16 +1721,17 @@ update_workspace_list (WnckScreen *screen)
   static int reentrancy_guard = 0;
 
   g_return_if_fail (reentrancy_guard == 0);
-  
+
   if (!screen->priv->need_update_workspace_list)
     return;
-  
+
   screen->priv->need_update_workspace_list = FALSE;
 
   ++reentrancy_guard;
-  
+
   n_spaces = 0;
-  if (!_wnck_get_cardinal (screen->priv->xroot,
+  if (!_wnck_get_cardinal (screen->priv->xscreen,
+                           screen->priv->xroot,
                            _wnck_atom_get ("_NET_NUMBER_OF_DESKTOPS"),
                            &n_spaces))
     n_spaces = 1;
@@ -1827,12 +1741,12 @@ update_workspace_list (WnckScreen *screen)
       g_warning ("Someone set a weird number of desktops in _NET_NUMBER_OF_DESKTOPS, assuming the value is 1\n");
       n_spaces = 1;
     }
-  
+
   old_n_spaces = g_list_length (screen->priv->workspaces);
 
   deleted = NULL;
   created = NULL;
-  
+
   if (old_n_spaces == n_spaces)
     {
       --reentrancy_guard;
@@ -1852,7 +1766,7 @@ update_workspace_list (WnckScreen *screen)
   else
     {
       int i;
-      
+
       g_assert (old_n_spaces < n_spaces);
 
       /* Need to create some workspaces */
@@ -1867,7 +1781,7 @@ update_workspace_list (WnckScreen *screen)
                                                     space);
 
           created = g_list_prepend (created, space);
-          
+
           ++i;
         }
 
@@ -1887,26 +1801,26 @@ update_workspace_list (WnckScreen *screen)
           screen->priv->active_workspace = NULL;
           emit_active_workspace_changed (screen, space);
         }
-      
+
       emit_workspace_destroyed (screen, space);
-      
+
       tmp = tmp->next;
     }
-  
+
   tmp = created;
   while (tmp != NULL)
     {
       emit_workspace_created (screen, WNCK_WORKSPACE (tmp->data));
-      
+
       tmp = tmp->next;
     }
   g_list_free (created);
-  
+
   tmp = deleted;
   while (tmp != NULL)
     {
       g_object_unref (tmp->data);
-      
+
       tmp = tmp->next;
     }
   g_list_free (deleted);
@@ -1919,7 +1833,7 @@ update_workspace_list (WnckScreen *screen)
       screen->priv->need_update_active_workspace = TRUE;
       queue_update (screen);
     }
-  
+
   --reentrancy_guard;
 }
 
@@ -1933,7 +1847,7 @@ update_viewport_settings (WnckScreen *screen)
   gboolean do_update;
   int space_width, space_height;
   gboolean got_viewport_prop;
-  
+
   if (!screen->priv->need_update_viewport_settings)
     return;
 
@@ -1946,10 +1860,11 @@ update_viewport_settings (WnckScreen *screen)
   /* If no property, use the screen's size */
   space_width = wnck_screen_get_width (screen);
   space_height = wnck_screen_get_height (screen);
-  
+
   p_coord = NULL;
   n_coord = 0;
-  if (_wnck_get_cardinal_list (screen->priv->xroot,
+  if (_wnck_get_cardinal_list (screen->priv->xscreen,
+                               screen->priv->xroot,
 			       _wnck_atom_get ("_NET_DESKTOP_GEOMETRY"),
                                &p_coord, &n_coord) &&
       p_coord != NULL)
@@ -1958,31 +1873,32 @@ update_viewport_settings (WnckScreen *screen)
 	{
           space_width = p_coord[0];
           space_height = p_coord[1];
-          
+
           if (space_width < wnck_screen_get_width (screen))
             space_width = wnck_screen_get_width (screen);
 
           if (space_height < wnck_screen_get_height (screen))
             space_height = wnck_screen_get_height (screen);
 	}
-      
+
       g_free (p_coord);
     }
-          
+
   for (i = 0; i < n_spaces; i++)
     {
       space = wnck_screen_get_workspace (screen, i);
       g_assert (space != NULL);
-      
+
       if (_wnck_workspace_set_geometry (space, space_width, space_height))
         do_update = TRUE;
     }
 
   got_viewport_prop = FALSE;
-  
+
   p_coord = NULL;
   n_coord = 0;
-  if (_wnck_get_cardinal_list (screen->priv->xroot,
+  if (_wnck_get_cardinal_list (screen->priv->xscreen,
+                               screen->priv->xroot,
                                _wnck_atom_get ("_NET_DESKTOP_VIEWPORT"),
                                &p_coord, &n_coord) &&
       p_coord != NULL)
@@ -1992,10 +1908,10 @@ update_viewport_settings (WnckScreen *screen)
           int screen_width, screen_height;
 
           got_viewport_prop = TRUE;
-          
+
           screen_width = wnck_screen_get_width (screen);
           screen_height = wnck_screen_get_height (screen);
-          
+
 	  for (i = 0; i < n_spaces; i++)
 	    {
               int x = 2 * i;
@@ -2003,7 +1919,7 @@ update_viewport_settings (WnckScreen *screen)
 
               space = wnck_screen_get_workspace (screen, i);
               g_assert (space != NULL);
-              
+
 	      /* p_coord[x] is unsigned, and thus >= 0 */
               if (p_coord[x] > space_width - screen_width)
                 p_coord[x] = space_width - screen_width;
@@ -2017,7 +1933,7 @@ update_viewport_settings (WnckScreen *screen)
                 do_update = TRUE;
 	    }
 	}
-      
+
       g_free (p_coord);
     }
 
@@ -2027,12 +1943,12 @@ update_viewport_settings (WnckScreen *screen)
         {
           space = wnck_screen_get_workspace (screen, i);
           g_assert (space != NULL);
-          
+
           if (_wnck_workspace_set_viewport (space, 0, 0))
             do_update = TRUE;
         }
     }
-  
+
   if (do_update)
     emit_viewports_changed (screen);
 }
@@ -2043,23 +1959,24 @@ update_active_workspace (WnckScreen *screen)
   int number;
   WnckWorkspace *previous_space;
   WnckWorkspace *space;
-  
+
   if (!screen->priv->need_update_active_workspace)
     return;
 
   screen->priv->need_update_active_workspace = FALSE;
 
   number = 0;
-  if (!_wnck_get_cardinal (screen->priv->xroot,
+  if (!_wnck_get_cardinal (screen->priv->xscreen,
+                           screen->priv->xroot,
                            _wnck_atom_get ("_NET_CURRENT_DESKTOP"),
                            &number))
     number = -1;
-  
+
   space = wnck_screen_get_workspace (screen, number);
 
   if (space == screen->priv->active_workspace)
     return;
-  
+
   previous_space = screen->priv->active_workspace;
   screen->priv->active_workspace = space;
 
@@ -2071,14 +1988,15 @@ update_active_window (WnckScreen *screen)
 {
   WnckWindow *window;
   Window xwindow;
-  
+
   if (!screen->priv->need_update_active_window)
     return;
 
   screen->priv->need_update_active_window = FALSE;
-  
+
   xwindow = None;
-  _wnck_get_window (screen->priv->xroot,
+  _wnck_get_window (screen->priv->xscreen,
+                    screen->priv->xroot,
                     _wnck_atom_get ("_NET_ACTIVE_WINDOW"),
                     &xwindow);
 
@@ -2106,9 +2024,10 @@ update_workspace_layout (WnckScreen *screen)
 
   list = NULL;
   n_items = 0;
-  if (_wnck_get_cardinal_list (screen->priv->xroot,
+  if (_wnck_get_cardinal_list (screen->priv->xscreen,
+                               screen->priv->xroot,
                                _wnck_atom_get ("_NET_DESKTOP_LAYOUT"),
-                               &list, 
+                               &list,
                                &n_items))
     {
       if (n_items == 3 || n_items == 4)
@@ -2194,21 +2113,22 @@ update_workspace_names (WnckScreen *screen)
   int i;
   GList *tmp;
   GList *copy;
-  
+
   if (!screen->priv->need_update_workspace_names)
     return;
 
   screen->priv->need_update_workspace_names = FALSE;
 
-  names = _wnck_get_utf8_list (screen->priv->xroot,
+  names = _wnck_get_utf8_list (screen->priv->xscreen,
+                               screen->priv->xroot,
                                _wnck_atom_get ("_NET_DESKTOP_NAMES"));
 
   copy = g_list_copy (screen->priv->workspaces);
-  
+
   i = 0;
   tmp = copy;
   while (tmp != NULL)
-    {      
+    {
       if (names && names[i])
         {
           _wnck_workspace_update_name (tmp->data, names[i]);
@@ -2229,20 +2149,21 @@ static void
 update_bg_pixmap (WnckScreen *screen)
 {
   Pixmap p;
-  
+
   if (!screen->priv->need_update_bg_pixmap)
     return;
-  
+
   screen->priv->need_update_bg_pixmap = FALSE;
 
   p = None;
-  _wnck_get_pixmap (screen->priv->xroot,
+  _wnck_get_pixmap (screen->priv->xscreen,
+                    screen->priv->xroot,
                     _wnck_atom_get ("_XROOTPMAP_ID"),
                     &p);
   /* may have failed, so p may still be None */
 
   screen->priv->bg_pixmap = p;
-  
+
   emit_background_changed (screen);
 }
 
@@ -2250,41 +2171,44 @@ static void
 update_showing_desktop (WnckScreen *screen)
 {
   int showing_desktop;
-  
+
   if (!screen->priv->need_update_showing_desktop)
     return;
-  
+
   screen->priv->need_update_showing_desktop = FALSE;
 
   showing_desktop = FALSE;
-  _wnck_get_cardinal (screen->priv->xroot,
+  _wnck_get_cardinal (screen->priv->xscreen,
+                      screen->priv->xroot,
                       _wnck_atom_get ("_NET_SHOWING_DESKTOP"),
                       &showing_desktop);
 
   screen->priv->showing_desktop = showing_desktop != 0;
-  
+
   emit_showing_desktop_changed (screen);
 }
 
 static void
 update_wm (WnckScreen *screen)
 {
-  Window  wm_window;
-  
+  Window wm_window;
+
   if (!screen->priv->need_update_wm)
     return;
-  
+
   screen->priv->need_update_wm = FALSE;
 
   wm_window = None;
-  _wnck_get_window (screen->priv->xroot,
+  _wnck_get_window (screen->priv->xscreen,
+                    screen->priv->xroot,
                     _wnck_atom_get ("_NET_SUPPORTING_WM_CHECK"),
                     &wm_window);
 
   g_free (screen->priv->wm_name);
 
   if (wm_window != None)
-    screen->priv->wm_name = _wnck_get_utf8_property (wm_window,
+    screen->priv->wm_name = _wnck_get_utf8_property (screen->priv->xscreen,
+                                                     wm_window,
                                                      _wnck_atom_get ("_NET_WM_NAME"));
   else
     screen->priv->wm_name = NULL;
@@ -2311,7 +2235,7 @@ do_update_now (WnckScreen *screen)
       screen->priv->need_update_viewport_settings = TRUE;
       screen->priv->need_update_workspace_names = TRUE;
     }
-      
+
   /* First get our big-picture state in order */
   update_workspace_list (screen);
   update_client_list (screen);
@@ -2324,7 +2248,7 @@ do_update_now (WnckScreen *screen)
   update_workspace_names (screen);
   update_showing_desktop (screen);
   update_wm (screen);
-  
+
   update_bg_pixmap (screen);
 }
 
@@ -2338,7 +2262,7 @@ update_idle (gpointer data)
   screen->priv->update_handler = 0;
 
   do_update_now (screen);
-  
+
   return FALSE;
 }
 
@@ -2392,7 +2316,7 @@ emit_window_opened (WnckScreen *screen,
 {
   g_signal_emit (G_OBJECT (screen),
                  signals[WINDOW_OPENED],
-                 0, window);  
+                 0, window);
 }
 
 static void
@@ -2410,7 +2334,7 @@ emit_workspace_created (WnckScreen    *screen,
 {
   g_signal_emit (G_OBJECT (screen),
                  signals[WORKSPACE_CREATED],
-                 0, space);  
+                 0, space);
 }
 
 static void
@@ -2555,7 +2479,7 @@ gulong
 wnck_screen_get_background_pixmap (WnckScreen *screen)
 {
   g_return_val_if_fail (WNCK_IS_SCREEN (screen), None);
-  
+
   return screen->priv->bg_pixmap;
 }
 
@@ -2622,7 +2546,7 @@ _wnck_screen_get_workspace_layout (WnckScreen             *screen,
                                    _WnckLayoutCorner      *starting_corner)
 {
   g_return_if_fail (WNCK_IS_SCREEN (screen));
-  
+
   if (orientation)
     *orientation = screen->priv->vertical_workspaces ?
                        WNCK_LAYOUT_ORIENTATION_VERTICAL :
@@ -2673,12 +2597,12 @@ wnck_screen_try_set_workspace_layout (WnckScreen *screen,
                                       int         columns)
 {
   int retval;
-  
+
   g_return_val_if_fail (WNCK_IS_SCREEN (screen),
                         WNCK_NO_MANAGER_TOKEN);
   g_return_val_if_fail (rows != 0 || columns != 0,
                         WNCK_NO_MANAGER_TOKEN);
-  
+
   retval = _wnck_try_desktop_layout_manager (screen->priv->xscreen, current_token);
 
   if (retval != WNCK_NO_MANAGER_TOKEN)
@@ -2725,7 +2649,7 @@ gboolean
 wnck_screen_get_showing_desktop (WnckScreen *screen)
 {
   g_return_val_if_fail (WNCK_IS_SCREEN (screen), FALSE);
-  
+
   return screen->priv->showing_desktop;
 }
 
@@ -2769,8 +2693,8 @@ wnck_screen_move_viewport (WnckScreen *screen,
   g_return_if_fail (WNCK_IS_SCREEN (screen));
   g_return_if_fail (x >= 0);
   g_return_if_fail (y >= 0);
-  
-  _wnck_change_viewport (WNCK_SCREEN_XSCREEN (screen), x, y);
+
+  _wnck_change_viewport (screen->priv->xscreen, x, y);
 }
 
 #ifdef HAVE_STARTUP_NOTIFICATION
@@ -2778,7 +2702,7 @@ SnDisplay*
 _wnck_screen_get_sn_display (WnckScreen *screen)
 {
   g_return_val_if_fail (WNCK_IS_SCREEN (screen), NULL);
-  
+
   return screen->priv->sn_display;
 }
 #endif /* HAVE_STARTUP_NOTIFICATION */
@@ -2791,7 +2715,7 @@ _wnck_screen_change_workspace_name (WnckScreen *screen,
   int n_spaces;
   char **names;
   int i;
-  
+
   n_spaces = wnck_screen_get_workspace_count (screen);
 
   names = g_new0 (char*, n_spaces + 1);
@@ -2810,11 +2734,12 @@ _wnck_screen_change_workspace_name (WnckScreen *screen,
           else
             names[i] = (char*) ""; /* maybe this should be a g_warning() */
         }
-      
+
       ++i;
     }
 
-  _wnck_set_utf8_list (screen->priv->xroot,
+  _wnck_set_utf8_list (screen->priv->xscreen,
+                       screen->priv->xroot,
                        _wnck_atom_get ("_NET_DESKTOP_NAMES"),
                        names);
 
