@@ -77,10 +77,10 @@ G_DEFINE_TYPE (WnckSelector, wnck_selector, GTK_TYPE_MENU_BAR);
 static GObject *wnck_selector_constructor (GType                  type,
                                            guint                  n_construct_properties,
                                            GObjectConstructParam *construct_properties);
+static void wnck_selector_dispose           (GObject           *object);
 static void wnck_selector_finalize          (GObject           *object);
 static void wnck_selector_realize           (GtkWidget *widget);
 static void wnck_selector_unrealize         (GtkWidget *widget);
-static void wnck_selector_destroy           (GtkObject *object);
 static void wnck_selector_connect_to_window (WnckSelector      *selector,
                                              WnckWindow        *window);
 
@@ -156,7 +156,7 @@ wnck_selector_dimm_icon (GdkPixbuf *pixbuf)
   w = gdk_pixbuf_get_width (pixbuf);
   h = gdk_pixbuf_get_height (pixbuf);
 
-  if (gdk_pixbuf_get_has_alpha (pixbuf)) 
+  if (gdk_pixbuf_get_has_alpha (pixbuf))
     dimmed = gdk_pixbuf_copy (pixbuf);
   else
     dimmed = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
@@ -220,7 +220,7 @@ wnck_selector_set_window_icon (WnckSelector *selector,
     }
 
   if (window && wnck_window_is_minimized (window))
-    {      
+    {
       pixbuf = wnck_selector_dimm_icon (pixbuf);
       freeme2 = pixbuf;
     }
@@ -244,7 +244,7 @@ wnck_selector_set_active_window (WnckSelector *selector, WnckWindow *window)
 static void
 wnck_selector_make_menu_consistent (WnckSelector *selector)
 {
-  GList     *l;
+  GList     *l, *children;
   int        workspace_n;
   GtkWidget *workspace_item;
   GtkWidget *separator;
@@ -261,7 +261,9 @@ wnck_selector_make_menu_consistent (WnckSelector *selector)
 
   visible_window = FALSE;
 
-  for (l = GTK_MENU_SHELL (selector->priv->menu)->children; l; l = l->next)
+  children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+
+  for (l = children; l; l = l->next)
     {
       int i;
 
@@ -285,7 +287,7 @@ wnck_selector_make_menu_consistent (WnckSelector *selector)
           separator_is_last = TRUE;
           separator = GTK_WIDGET (l->data);
         }
-      else if (GTK_WIDGET_VISIBLE (l->data) &&
+      else if (gtk_widget_get_visible (l->data) &&
                l->data != selector->priv->no_windows_item)
         {
           separator_is_last = FALSE;
@@ -315,6 +317,8 @@ wnck_selector_make_menu_consistent (WnckSelector *selector)
         } /* end if (normal item) */
     }
 
+  g_list_free (children);
+
   /* do we have a trailing workspace item to be hidden? */
   if (workspace_item)
     gtk_widget_hide (workspace_item);
@@ -420,8 +424,8 @@ wnck_selector_window_state_changed (WnckWindow *window,
       gtk_menu_reposition (GTK_MENU (selector->priv->menu));
     }
 
-  if (changed_mask & 
-      (WNCK_WINDOW_STATE_DEMANDS_ATTENTION | WNCK_WINDOW_STATE_URGENT))      
+  if (changed_mask &
+      (WNCK_WINDOW_STATE_DEMANDS_ATTENTION | WNCK_WINDOW_STATE_URGENT))
     {
       if (wnck_window_or_transient_needs_attention (window))
 	_make_gtk_label_bold (GTK_LABEL (item->label));
@@ -447,7 +451,7 @@ wnck_selector_window_workspace_changed (WnckWindow   *window,
 
   item = NULL;
 
-  if (!selector->priv->menu || !GTK_WIDGET_VISIBLE (selector->priv->menu))
+  if (!selector->priv->menu || !gtk_widget_get_visible (selector->priv->menu))
     return;
 
   if (!selector->priv->window_hash)
@@ -505,6 +509,7 @@ wnck_selector_activate_window (WnckWindow *window)
 static gint
 wnck_selector_get_width (GtkWidget *widget, const char *text)
 {
+  GtkStyle *style;
   PangoContext *context;
   PangoFontMetrics *metrics;
   gint char_width;
@@ -515,9 +520,10 @@ wnck_selector_get_width (GtkWidget *widget, const char *text)
   gint width;
 
   gtk_widget_ensure_style (widget);
+  style = gtk_widget_get_style (widget);
 
   context = gtk_widget_get_pango_context (widget);
-  metrics = pango_context_get_metrics (context, widget->style->font_desc,
+  metrics = pango_context_get_metrics (context, style->font_desc,
                                        pango_context_get_language (context));
   char_width = pango_font_metrics_get_approximate_char_width (metrics);
   pango_font_metrics_unref (metrics);
@@ -535,7 +541,7 @@ wnck_selector_get_width (GtkWidget *widget, const char *text)
   return width;
 }
 
-static void  
+static void
 wnck_selector_drag_begin (GtkWidget          *widget,
 			  GdkDragContext     *context,
 			  WnckWindow         *window)
@@ -548,14 +554,14 @@ wnck_selector_drag_begin (GtkWidget          *widget,
       if (GTK_IS_MENU (widget))
         widget = gtk_menu_get_attach_widget (GTK_MENU (widget));
       else
-        widget = widget->parent;
+        widget = gtk_widget_get_parent (widget);
     }
 
   if (widget)
     _wnck_window_set_as_drag_icon (window, context, widget);
 }
 
-static void  
+static void
 wnck_selector_drag_data_get (GtkWidget          *widget,
 			     GdkDragContext     *context,
 			     GtkSelectionData   *selection_data,
@@ -563,11 +569,11 @@ wnck_selector_drag_data_get (GtkWidget          *widget,
 			     guint               time,
 			     WnckWindow         *window)
 {
-  gulong xid;    
+  gulong xid;
 
   xid = wnck_window_get_xid (window);
   gtk_selection_data_set (selection_data,
- 		          selection_data->target,
+                          gtk_selection_data_get_target (selection_data),
 			  8, (guchar *)&xid, sizeof (gulong));
 }
 
@@ -620,35 +626,28 @@ wnck_selector_item_new (WnckSelector *selector,
       g_signal_connect_object (item, "drag_data_get",
                                G_CALLBACK (wnck_selector_drag_data_get),
                                G_OBJECT (window),
-                               0); 
+                               0);
 
       g_signal_connect_object (item, "drag_begin",
                                G_CALLBACK (wnck_selector_drag_begin),
                                G_OBJECT (window),
-                               0); 
+                               0);
     }
 
   return item;
 }
 
-static gboolean
-wnck_selector_workspace_label_exposed (GtkWidget *widget)
-{
-  /* Bad hack to make the label draw normally, instead of insensitive. */
-  widget->state = GTK_STATE_NORMAL;
-
-  return FALSE;
-}
-
 static void
 wnck_selector_workspace_name_changed (WnckWorkspace *workspace,
                                       GtkLabel      *label)
 {
+  GtkStyle *style;
   GdkColor *color;
   char     *name;
   char     *markup;
 
-  color = &GTK_WIDGET (label)->style->fg[GTK_STATE_INSENSITIVE];
+  style = gtk_widget_get_style (GTK_WIDGET (label));
+  color = &style->fg[GTK_STATE_INSENSITIVE];
 
   name = g_markup_escape_text (wnck_workspace_get_name (workspace), -1);
   markup = g_strdup_printf ("<span size=\"x-small\" style=\"italic\" foreground=\"#%.2x%.2x%.2x\">%s</span>",
@@ -678,14 +677,13 @@ wnck_selector_add_workspace (WnckSelector *selector,
 
   workspace = wnck_screen_get_workspace (screen, workspace_n);
 
-  item = gtk_menu_item_new ();
-  gtk_widget_set_sensitive (item, FALSE);
+  /* We use a separator in which we add a label. This makes the menu item not
+   * selectable without any hack. */
+  item = gtk_separator_menu_item_new ();
 
   label = gtk_label_new ("");
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);
-  g_signal_connect (G_OBJECT (label), "expose-event",
-                    G_CALLBACK (wnck_selector_workspace_label_exposed), NULL);
   /* the handler will also take care of setting the name for the first time,
    * and we'll be able to adapt to theme changes */
   g_signal_connect (G_OBJECT (label), "style-set",
@@ -761,21 +759,23 @@ wnck_selector_insert_window (WnckSelector *selector, WnckWindow *window)
     {
       /* window is pinned or in the current workspace
        * => insert before the separator */
-      GList *l;
+      GList *l, *children;
 
       i = 0;
 
-      for (l = GTK_MENU_SHELL (selector->priv->menu)->children; l; l = l->next)
+      children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+      for (l = children; l; l = l->next)
         {
           if (GTK_IS_SEPARATOR_MENU_ITEM (l->data))
             break;
           i++;
         }
+      g_list_free (children);
 
       gtk_menu_shell_insert (GTK_MENU_SHELL (selector->priv->menu),
                              item, i);
     }
-  else 
+  else
     {
       workspace_n = wnck_workspace_get_number (workspace);
 
@@ -785,12 +785,12 @@ wnck_selector_insert_window (WnckSelector *selector, WnckWindow *window)
       else
         {
           /* insert just before the next workspace item */
-          GList *l;
+          GList *l, *children;
 
           i = 0;
 
-          for (l = GTK_MENU_SHELL (selector->priv->menu)->children;
-               l; l = l->next)
+          children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+          for (l = children; l; l = l->next)
             {
               int j;
               j = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (l->data),
@@ -799,6 +799,7 @@ wnck_selector_insert_window (WnckSelector *selector, WnckWindow *window)
                 break;
               i++;
             }
+          g_list_free (children);
 
           gtk_menu_shell_insert (GTK_MENU_SHELL (selector->priv->menu),
                                  item, i);
@@ -821,7 +822,7 @@ wnck_selector_window_opened (WnckScreen *screen,
 {
   wnck_selector_connect_to_window (selector, window);
 
-  if (!selector->priv->menu || !GTK_WIDGET_VISIBLE (selector->priv->menu))
+  if (!selector->priv->menu || !gtk_widget_get_visible (selector->priv->menu))
     return;
 
   if (!selector->priv->window_hash)
@@ -842,7 +843,7 @@ wnck_selector_window_closed (WnckScreen *screen,
   if (window == selector->priv->icon_window)
     wnck_selector_set_active_window (selector, NULL);
 
-  if (!selector->priv->menu || !GTK_WIDGET_VISIBLE (selector->priv->menu))
+  if (!selector->priv->menu || !gtk_widget_get_visible (selector->priv->menu))
     return;
 
   if (!selector->priv->window_hash)
@@ -865,7 +866,7 @@ wnck_selector_workspace_created (WnckScreen    *screen,
                                  WnckWorkspace *workspace,
                                  WnckSelector  *selector)
 {
-  if (!selector->priv->menu || !GTK_WIDGET_VISIBLE (selector->priv->menu))
+  if (!selector->priv->menu || !gtk_widget_get_visible (selector->priv->menu))
     return;
 
   /* this is assuming that the new workspace will have a higher number
@@ -884,11 +885,11 @@ wnck_selector_workspace_destroyed (WnckScreen    *screen,
                                    WnckWorkspace *workspace,
                                    WnckSelector  *selector)
 {
-  GList     *l;
+  GList     *l, *children;
   GtkWidget *destroy;
   int        i;
 
-  if (!selector->priv->menu || !GTK_WIDGET_VISIBLE (selector->priv->menu))
+  if (!selector->priv->menu || !gtk_widget_get_visible (selector->priv->menu))
     return;
 
   destroy = NULL;
@@ -896,7 +897,9 @@ wnck_selector_workspace_destroyed (WnckScreen    *screen,
   i = wnck_workspace_get_number (workspace);
 
   /* search for the item of this workspace so that we destroy it */
-  for (l = GTK_MENU_SHELL (selector->priv->menu)->children; l; l = l->next)
+  children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+
+  for (l = children; l; l = l->next)
     {
       int j;
 
@@ -912,6 +915,8 @@ wnck_selector_workspace_destroyed (WnckScreen    *screen,
                            GINT_TO_POINTER (j - 1));
     }
 
+  g_list_free (children);
+
   if (destroy)
     gtk_widget_destroy (destroy);
 
@@ -1084,7 +1089,7 @@ wnck_selector_scroll_cb (WnckSelector *selector,
 
       previous_window = window;
     }
-  
+
   return TRUE;
 }
 
@@ -1241,16 +1246,14 @@ static void
 wnck_selector_class_init (WnckSelectorClass *klass)
 {
   GObjectClass   *object_class     = G_OBJECT_CLASS (klass);
-  GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class     = GTK_WIDGET_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (WnckSelectorPrivate));
 
   object_class->constructor = wnck_selector_constructor;
+  object_class->dispose = wnck_selector_dispose;
   object_class->finalize = wnck_selector_finalize;
 
-  gtk_object_class->destroy = wnck_selector_destroy;
-
   widget_class->realize   = wnck_selector_realize;
   widget_class->unrealize = wnck_selector_unrealize;
 }
@@ -1287,7 +1290,7 @@ wnck_selector_finalize (GObject *object)
 }
 
 static void
-wnck_selector_destroy (GtkObject *object)
+wnck_selector_dispose (GObject *object)
 {
   WnckSelector *selector;
 
@@ -1300,7 +1303,7 @@ wnck_selector_destroy (GtkObject *object)
   selector->priv->image       = NULL;
   selector->priv->icon_window = NULL;
 
-  GTK_OBJECT_CLASS (wnck_selector_parent_class)->destroy (object);
+  G_OBJECT_CLASS (wnck_selector_parent_class)->dispose (object);
 }
 
 static void
