@@ -21,8 +21,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#undef WNCK_DISABLE_DEPRECATED
-
 #include <config.h>
 
 #include <glib/gi18n-lib.h>
@@ -53,26 +51,9 @@
  * @stability: Unstable
  *
  * These functions are utility functions providing some additional features to
- * libwcnk users.
+ * libwnck users.
  */
 
-/**
- * wnck_gtk_window_set_dock_type:
- * @window: a <classname>GtkWindow</classname>.
- *
- * Sets the semantic type of @window to %WNCK_WINDOW_DOCK.
- *
- * Deprecated:2.20: Use gdk_window_set_type_hint() instead.
- */
-void
-wnck_gtk_window_set_dock_type (GtkWindow *window)
-{
-  g_return_if_fail (GTK_IS_WINDOW (window));
-
-  gdk_window_set_type_hint (gtk_widget_get_window (GTK_WIDGET (window)),
-		  	    GDK_WINDOW_TYPE_HINT_DOCK);
-}
-
 typedef enum
 {
   WNCK_EXT_UNKNOWN = 0,
@@ -101,16 +82,16 @@ _wnck_print_resource_usage (WnckResourceUsage *usage)
            "\tunknowns      : %d\n"
            "\tpixmap bytes  : %ld\n"
            "\ttotal bytes   : ~%ld\n",
-           usage->n_windows, 
-           usage->n_gcs, 
+           usage->n_windows,
+           usage->n_gcs,
            usage->n_fonts,
-           usage->n_pixmaps,  
+           usage->n_pixmaps,
            usage->n_pictures,
            usage->n_glyphsets,
            usage->n_colormap_entries,
            usage->n_passive_grabs,
            usage->n_cursors,
-           usage->n_other, 
+           usage->n_other,
            usage->pixmap_bytes,
            usage->total_bytes_estimate);
 }
@@ -138,7 +119,7 @@ wnck_init_resource_usage (GdkDisplay *gdisplay)
 #else
       status = WNCK_EXT_MISSING;
 #endif
-      
+
       g_object_set_data (G_OBJECT (gdisplay),
                          "wnck-xres-status",
                          GINT_TO_POINTER (status));
@@ -192,11 +173,11 @@ wnck_xid_read_resource_usage (GdkDisplay        *gdisplay,
    Atom colormap_entry_atom;
    Atom passive_grab_atom;
    Atom cursor_atom;
-   
+
    types = NULL;
    n_types = 0;
    pixmap_bytes = 0;
-   
+
   _wnck_error_trap_push ();
 
   xdisplay = GDK_DISPLAY_XDISPLAY (gdisplay);
@@ -207,8 +188,8 @@ wnck_xid_read_resource_usage (GdkDisplay        *gdisplay,
 
    XResQueryClientPixmapBytes (xdisplay,
                                xid, &pixmap_bytes);
-   _wnck_error_trap_pop ();   
-   
+   _wnck_error_trap_pop ();
+
    usage->pixmap_bytes = pixmap_bytes;
 
    pixmap_atom = _wnck_atom_get ("PIXMAP");
@@ -220,12 +201,12 @@ wnck_xid_read_resource_usage (GdkDisplay        *gdisplay,
    colormap_entry_atom = _wnck_atom_get ("COLORMAP ENTRY");
    passive_grab_atom = _wnck_atom_get ("PASSIVE GRAB");
    cursor_atom = _wnck_atom_get ("CURSOR");
-   
+
    i = 0;
    while (i < n_types)
      {
        int t = types[i].resource_type;
-       
+
        if (t == pixmap_atom)
          usage->n_pixmaps += types[i].count;
        else if (t == window_atom)
@@ -246,7 +227,7 @@ wnck_xid_read_resource_usage (GdkDisplay        *gdisplay,
          usage->n_cursors += types[i].count;
        else
          usage->n_other += types[i].count;
-       
+
        ++i;
      }
 
@@ -297,12 +278,13 @@ wnck_gulong_equal (gconstpointer a,
 }
 
 static gulong
-wnck_check_window_for_pid (Window win,
-                           XID    match_xid,
-                           XID    mask)
+wnck_check_window_for_pid (Screen *screen,
+                           Window  win,
+                           XID     match_xid,
+                           XID     mask)
 {
   if ((win & ~mask) == match_xid) {
-    return _wnck_get_pid (win);
+    return _wnck_get_pid (screen, win);
   }
 
   return 0;
@@ -310,6 +292,7 @@ wnck_check_window_for_pid (Window win,
 
 static void
 wnck_find_pid_for_resource_r (Display *xdisplay,
+                              Screen  *screen,
                               Window   win_top,
                               XID      match_xid,
                               XID      mask,
@@ -327,7 +310,7 @@ wnck_find_pid_for_resource_r (Display *xdisplay,
   while (gtk_events_pending ())
     gtk_main_iteration ();
 
-  found_pid = wnck_check_window_for_pid (win_top, match_xid, mask);
+  found_pid = wnck_check_window_for_pid (screen, win_top, match_xid, mask);
   if (found_pid != 0)
     {
       *xid = win_top;
@@ -337,14 +320,14 @@ wnck_find_pid_for_resource_r (Display *xdisplay,
   _wnck_error_trap_push ();
   qtres = XQueryTree (xdisplay, win_top, &dummy, &dummy,
                       &children, &n_children);
-  err = _wnck_error_trap_pop ();   
+  err = _wnck_error_trap_pop ();
 
   if (!qtres || err != Success)
     return;
 
-  for (i = 0; i < n_children; i++) 
+  for (i = 0; i < n_children; i++)
     {
-      wnck_find_pid_for_resource_r (xdisplay, children[i],
+      wnck_find_pid_for_resource_r (xdisplay, screen, children[i],
                                     match_xid, mask, xid, pid);
 
       if (*pid != 0)
@@ -420,14 +403,16 @@ wnck_pid_read_resource_usage_fill_cache (struct xresclient_state *state)
 
   for (i = 0; i < ScreenCount (state->xdisplay); i++)
     {
-      Window root;
+      Screen *screen;
+      Window  root;
 
+      screen = ScreenOfDisplay (state->xdisplay, i);
       root = RootWindow (state->xdisplay, i);
 
       if (root == None)
         continue;
 
-      wnck_find_pid_for_resource_r (state->xdisplay, root, match_xid,
+      wnck_find_pid_for_resource_r (state->xdisplay, screen, root, match_xid,
                                     state->clients[state->next].resource_mask,
                                     &xid, &pid);
 
@@ -466,7 +451,7 @@ wnck_pid_read_resource_usage_start_build_cache (GdkDisplay *gdisplay)
   xdisplay = GDK_DISPLAY_XDISPLAY (gdisplay);
 
   _wnck_error_trap_push ();
-  XResQueryClients (xdisplay, &xres_state.n_clients, &xres_state.clients); 
+  XResQueryClients (xdisplay, &xres_state.n_clients, &xres_state.clients);
   err = _wnck_error_trap_pop ();
 
   if (err != Success)
@@ -562,7 +547,7 @@ wnck_pid_read_resource_usage_no_cache (GdkDisplay        *gdisplay,
       WnckScreen *screen;
       GList *windows;
       GList *tmp;
-      
+
       screen = wnck_screen_get (i);
 
       g_assert (screen != NULL);
@@ -583,7 +568,7 @@ wnck_pid_read_resource_usage_no_cache (GdkDisplay        *gdisplay,
 
           tmp = tmp->next;
         }
-      
+
       ++i;
     }
 }
@@ -778,7 +763,7 @@ _wnck_stock_icons_init (void)
     return;
 
   done = TRUE;
-  
+
   factory = gtk_icon_factory_new ();
   gtk_icon_factory_add_default (factory);
 
@@ -794,7 +779,7 @@ _wnck_stock_icons_init (void)
       icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
       gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
       gtk_icon_set_unref (icon_set);
-		
+
       g_object_unref (G_OBJECT (pixbuf));
     }
 
